@@ -38,11 +38,14 @@ func main() {
 		os.Exit(1)
 	}
 
-	config, err := geoipupdate.NewConfig(args.ConfigFile, defaultDatabaseDirectory, args.DatabaseDirectory, args.Verbose)
+	config, err := geoipupdate.NewConfig(
+		args.ConfigFile, defaultDatabaseDirectory, args.DatabaseDirectory, args.Verbose)
 	if err != nil {
-		fatalLogger("error loading configuration file", err)
+		fatalLogger(fmt.Sprintf("error loading configuration file %s", args.ConfigFile), err)
 	}
+
 	if config.Verbose {
+		log.Printf("geoipupdate version %s", version)
 		log.Printf("Using config file %s", args.ConfigFile)
 		log.Printf("Using database directory %s", config.DatabaseDirectory)
 	}
@@ -60,15 +63,15 @@ func run(client *http.Client, config *geoipupdate.Config) error {
 	for _, editionID := range config.EditionIDs {
 		filename, err := geoipupdate.GetFilename(config, editionID, client)
 		if err != nil {
-			return errors.Wrap(err, "error retrieving filename")
+			return errors.Wrapf(err, "error retrieving filename for %s", editionID)
 		}
 		filePath := filepath.Join(config.DatabaseDirectory, filename)
 		dbWriter, err := database.NewLocalFileDatabaseWriter(filePath, config.LockFile, config.Verbose)
 		if err != nil {
-			return errors.Wrap(err, "error creating database writer")
+			return errors.Wrapf(err, "error creating database writer for %s", editionID)
 		}
 		if err := dbReader.Get(dbWriter, editionID); err != nil {
-			return err
+			return errors.WithMessagef(err, "error while getting database for %s", editionID)
 		}
 	}
 	return nil
